
-- PERFORMANCE MEASUREMENT : exception raise and handle time
--                           exception in loop

with REMOTE_GLOBAL ; use REMOTE_GLOBAL ; -- control optimization
with ITERATION ; -- obtain stable measurement
with PIWG_IO ; -- output results

procedure E000001 is  -- main procedure to execute

  CPU_TIME : DURATION ; -- CPU time for one feature execution
  WALL_TIME : DURATION ; -- WALL time for one feature execution
  CHECK_TIMES : constant := 100 ; -- inside loop count and check
  ITERATION_COUNT : INTEGER ; -- set and varied by ITERATION package
  STABLE : BOOLEAN ; -- true when measurement stable
  MY_EXCEPTION : exception ;

--

begin

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        begin
          if A_ONE = 1 then
            GLOBAL := GLOBAL + A_ONE ;
            REMOTE ;
          else
            GLOBAL := GLOBAL + A_ONE ;
            REMOTE ;
            raise MY_EXCEPTION ;
          end if ;
        exception
          when MY_EXCEPTION =>
            null ;
        end ;
      end loop ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;


--
-- Test loop
--
-- establish exception raise and handle time

    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        begin
          if A_ONE = 2 then
            GLOBAL := GLOBAL + A_ONE ;
            REMOTE ;
          else
            GLOBAL := GLOBAL + A_ONE ;
            REMOTE ;
            raise MY_EXCEPTION ;
          end if ;
        exception
          when MY_EXCEPTION =>
            null ;
        end ;
      end loop ;
    end loop ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , STABLE ) ;
    exit when STABLE ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;


--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "E000001" , "Exception" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " Time to raise and handle an exception " ,
     " Exception defined locally and handled locally " ,
     " " ) ;


end E000001 ;
