
-- PERFORMANCE MEASUREMENT : function GET_LINE timing

with TEXT_IO ; use TEXT_IO ;
with REMOTE_GLOBAL ; use REMOTE_GLOBAL ; -- control optimization
with ITERATION ; -- obtain stable measurement
with PIWG_IO ; -- output results

procedure G000001 is  -- main procedure to execute
  CPU_TIME : DURATION ; -- CPU time for one feature execution
  WALL_TIME : DURATION ; -- WALL time for one feature execution
  CHECK_TIMES : constant := 100 ; -- inside loop count and check
  ITERATION_COUNT : INTEGER ; -- set and varied by ITERATION package
  STABLE : BOOLEAN ; -- true when measurement stable
  SCRATCH_FILE : FILE_TYPE ;
  SCRATCH_STRING : STRING ( 1..20 ) := "abcdefghijklmnopqrst";
  SCRATCH_LENGTH : NATURAL ;

begin
--
--  Build file for " GET_LINE " test
--
  CREATE ( SCRATCH_FILE , OUT_FILE , "SCRATCH1" ) ;
  for INSIDE_LOOP in 1 .. CHECK_TIMES loop
    PUT_LINE ( SCRATCH_FILE , SCRATCH_STRING ) ;
  end loop ;
  CLOSE ( SCRATCH_FILE ) ;
  OPEN ( SCRATCH_FILE , IN_FILE , "SCRATCH1" ) ;

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ; 
        REMOTE ;                   
      end loop ;
      RESET ( SCRATCH_FILE ) ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;

--
-- Test loop
--
-- establish the time for the GET_LINE pocdedure on 20 character strings

    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ; 
        REMOTE ;                   
        GET_LINE ( SCRATCH_FILE , SCRATCH_STRING , SCRATCH_LENGTH ) ;
      end loop ;
      RESET ( SCRATCH_FILE ) ;
    end loop ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , STABLE ) ;
    exit when STABLE ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;

  CLOSE ( SCRATCH_FILE ) ;
--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "G000001" , "Input/Output" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " TEXT_IO.GET_LINE reading 20 characters, time measured " ,
     " A scratch file is written, then read and reset " ,
     " " ) ;

end G000001 ;
