
-- PERFORMANCE MEASUREMENT : function GET timing ( INTERNAL )

with TEXT_IO ; use TEXT_IO ;
with REMOTE_GLOBAL ; use REMOTE_GLOBAL ; -- control optimization
with ITERATION ; -- obtain stable measurement
with PIWG_IO ; -- output results

procedure G000005 is  -- main procedure to execute
  CPU_TIME : DURATION ; -- CPU time for one feature execution
  WALL_TIME : DURATION ; -- WALL time for one feature execution
  CHECK_TIMES : constant := 100 ; -- inside loop count and check
  ITERATION_COUNT : INTEGER ; -- set and varied by ITERATION package
  STABLE : BOOLEAN ; -- true when measurement stable
  SCRATCH_STRING : STRING ( 1..6 ) := "100   ";
  SCRATCH_LENGTH : NATURAL ;
  SCRATCH_VALUE  : INTEGER ;
  package INT_IO is new INTEGER_IO ( INTEGER ) ; use INT_IO ;

begin

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ; 
        REMOTE ;                   
        PUT ( SCRATCH_STRING , INSIDE_LOOP ) ;
      end loop ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;


--
-- Test loop
--
-- establish the time for the GET pocdedure on an integer

    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ; 
        REMOTE ;                   
        PUT ( SCRATCH_STRING , INSIDE_LOOP ) ;
        GET ( SCRATCH_STRING , SCRATCH_VALUE , SCRATCH_LENGTH ) ;
      end loop ;
    end loop ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , STABLE ) ;
    exit when STABLE ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;

--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "G000005" , "Input/Output" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " TEXT_IO.GET an integer from a local string, timing measurement" ,
     " use TEXT_IO.PUT to convert 1..100 to a string " ,
     " then use TEXT_IO.GET to get the number back" ) ;

end G000005 ;
