-- PERFORMANCE MEASUREMENT : Minimum procedure call and return time
--                           procedure local
--                           no parameters

with REMOTE_GLOBAL ; use REMOTE_GLOBAL ;
with ITERATION ;
with PIWG_IO ;

procedure P000001 is  -- main procedure to execute

  CPU_TIME : DURATION ;
  WALL_TIME : DURATION ;
  CHECK_TIMES : constant := 100 ;
  ITERATION_COUNT : INTEGER ;
  ITS_OK : BOOLEAN ;

--

  procedure PROC_0 is  -- may be inlined thus zero time
  begin
    GLOBAL := GLOBAL + A_ONE ;
    REMOTE ;
  end ;

begin

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ;
        REMOTE ;
      end loop ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;

--
-- Test loop
--
    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        PROC_0 ;  -- this has control global increment and call inside
      end loop ;
    end loop ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , ITS_OK ) ;
    exit when ITS_OK ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;

--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "P000001" , "Procedure" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " Procedure call and return time ( may be zero if automatic inlining ) " ,
     " procedure is local " ,
     " no parameters " ) ;

end P000001 ;
