
-- PERFORMANCE MEASUREMENT : procedure call and return time
--                           procedure in package
--                           one "in" parameter

package PROC_PACKAGE_5 is
  procedure PROC_0 ( A : in INTEGER ) ;
end PROC_PACKAGE_5 ;

with PROC_PACKAGE_5 ; use PROC_PACKAGE_5 ;
with REMOTE_GLOBAL ; use REMOTE_GLOBAL ; -- control optimization
with ITERATION ; -- obtain stable measurement
with PIWG_IO ; -- output results

procedure P000005 is  -- main procedure to execute
  CPU_TIME : DURATION ; -- CPU time for one feature execution
  WALL_TIME : DURATION ; -- WALL time for one feature execution
  CHECK_TIMES : constant := 100 ; -- inside loop count and check
  ITERATION_COUNT : INTEGER ; -- set and varied by ITERATION package
  STABLE : BOOLEAN ; -- true when measurement stable

begin

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ; 
        REMOTE ;                   
      end loop ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;

--
-- Test loop
--
    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        PROC_0 ( A_ONE ) ; -- this has control global increment and call inside
      end loop ;
    end loop ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , STABLE ) ;
    exit when STABLE ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;

--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "P000005" , "Procedure" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " Procedure call and return time measurement" ,
     " The procedure is in a separately compiled package " ,
     " One parameter, in INTEGER " ) ;

end P000005 ;


with REMOTE_GLOBAL ; use REMOTE_GLOBAL ;
package body PROC_PACKAGE_5 is -- compare to P000003
  procedure PROC_0 ( A : in INTEGER ) is
  begin
    GLOBAL := GLOBAL + A ;
    REMOTE ;
  end ;
end PROC_PACKAGE_5 ;

