
-- PERFORMANCE MEASUREMENT : procedure call and return time
--                           procedure in package
--                           ten composite "in" parameters

package PROC_PACKAGE_12 is
  type MY_RECORD is
    record
      R1 : INTEGER ;
      R2 : INTEGER ;
      R3 : INTEGER ;
    end record ;
  procedure PROC_0 ( A1, A2, A3, A4, A5, A6, A7, A8, A9, A10 : in MY_RECORD ) ;
end PROC_PACKAGE_12 ;

with PROC_PACKAGE_12 ; use PROC_PACKAGE_12 ;
with REMOTE_GLOBAL ; use REMOTE_GLOBAL ; -- control optimization
with ITERATION ; -- obtain stable measurement
with PIWG_IO ; -- output results

procedure P000012 is  -- main procedure to execute
  CPU_TIME : DURATION ; -- CPU time for one feature execution
  WALL_TIME : DURATION ; -- WALL time for one feature execution
  CHECK_TIMES : constant := 100 ; -- inside loop count and check
  ITERATION_COUNT : INTEGER ; -- set and varied by ITERATION package
  STABLE : BOOLEAN ; -- true when measurement stable
  A : MY_RECORD := ( A_ONE , A_ONE + A_ONE , A_ONE + A_ONE + A_ONE ) ;

begin

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A.R1+A.R2+A.R3-A.R1-A.R2-A.R3+A.R1+A.R2
                          -A.R3+A.R1 ;
        REMOTE ;                   
      end loop ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;

--
-- Test loop
--
    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        PROC_0 ( A, A, A, A, A, A, A, A, A, A ) ;
        -- this has control global increment and call inside
      end loop ;
    end loop ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , STABLE ) ;
    exit when STABLE ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;

--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "P000012" , "Procedure" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " Procedure call and return time measurement" ,
     " Compare with P000010 ( discrete vs composite parameters ) " ,
     " 10 paramaters, in MY_RECORD  a three component record" ) ;

end P000012 ;


with REMOTE_GLOBAL ; use REMOTE_GLOBAL ;
package body PROC_PACKAGE_12 is -- compare to P00000
  procedure PROC_0 ( A1, A2, A3, A4, A5, A6, A7, A8, A9, A10 : in MY_RECORD ) is
  begin
    GLOBAL := GLOBAL + A1.R1+A2.R2+A3.R3-A4.R1-A5.R2-A6.R3+A7.R1+A8.R2
                      -A9.R3+A10.R1 ;
    REMOTE ;
  end ;
end PROC_PACKAGE_12 ;

