
-- PERFORMANCE MEASUREMENT : task entry call and return time, 1 task 1 entry
--                           task in package, no select

with REMOTE_GLOBAL ; use REMOTE_GLOBAL ;
package TASK_PACK_2 is
  task T1 is
    entry E1 ;
  end T1 ;
end TASK_PACK_2 ;

with TASK_PACK_2 ; use TASK_PACK_2 ;
with REMOTE_GLOBAL ; use REMOTE_GLOBAL ; -- control optimization
with ITERATION ; -- obtain stable measurement
with PIWG_IO ; -- output results

procedure T000002 is  -- main procedure to execute
  CPU_TIME : DURATION ; -- CPU time for one feature execution
  WALL_TIME : DURATION ; -- WALL time for one feature execution
  CHECK_TIMES : constant := 100 ; -- inside loop count and check
  ITERATION_COUNT : INTEGER ; -- set and varied by ITERATION package
  STABLE : BOOLEAN ; -- true when measurement stable

begin

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ; 
        REMOTE ;                   
      end loop ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;

--
-- Test loop
--
    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        T1.E1 ; -- this has control global increment and call inside
      end loop ;
    end loop ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , STABLE ) ;
    exit when STABLE ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;

--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "T000002" , "Tasking" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " Task entry call and return time measured" ,
     " One task active, one entry in task, task in a package " ,
     " no select statement " ) ;

  abort T1 ;
end T000002 ;

package body TASK_PACK_2 is
  task body T1 is
  begin
    loop
      accept E1 do
        GLOBAL := GLOBAL + A_ONE ;
        REMOTE ;
      end E1 ;
    end loop ;
  end ;
end TASK_PACK_2 ;
