-- PERFORMANCE MEASUREMENT : tasks entry call and return time
--                           1 task 2 entries
--                           one select statement

with REMOTE_GLOBAL ; use REMOTE_GLOBAL ;
package TASK_PACK_4 is
  task T1 is
    entry E1 ;
    entry E2 ;
  end T1 ;
end TASK_PACK_4 ;

with TASK_PACK_4 ; use TASK_PACK_4 ;
with REMOTE_GLOBAL ; use REMOTE_GLOBAL ; -- control optimization
with ITERATION ; -- obtain stable measurement
with PIWG_IO ; -- output results

procedure T000004 is  -- main procedure to execute
  CPU_TIME : DURATION ; -- CPU time for one feature execution
  WALL_TIME : DURATION ; -- WALL time for one feature execution
  CHECK_TIMES : constant := 100 ; -- inside loop count and check
  ITERATION_COUNT : INTEGER ; -- set and varied by ITERATION package
  STABLE : BOOLEAN ; -- true when measurement stable
  CASE_COUNT : constant := 2 ;

begin

  ITERATION.START_CONTROL ;  -- dummy to bring in pages on some machines

  delay 0.5 ;  -- wait for stable enviornment on some machines

  ITERATION.INITIALIZE ( ITERATION_COUNT ) ;

  loop  -- until stable measurement, ITERATION_COUNT increases each time

--
-- Control loop
--
    ITERATION.START_CONTROL ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        GLOBAL := GLOBAL + A_ONE ; 
        REMOTE ;                   
      end loop ;
    end loop ;
    ITERATION.STOP_CONTROL ( GLOBAL , CHECK_TIMES ) ;

--
-- Test loop
--
    ITERATION.START_TEST ;
    for J in 1 .. ITERATION_COUNT loop
      GLOBAL := 0 ;
      for INSIDE_LOOP in 1 .. CHECK_TIMES loop
        T1.E1 ; -- this has control global increment and call inside
        T1.E2 ; -- this has control global increment and call inside
      end loop ;
    end loop ;
    GLOBAL := GLOBAL / CASE_COUNT ;
    ITERATION.STOP_TEST ( GLOBAL , CHECK_TIMES ) ;
    ITERATION.TEST_STABLE ( ITERATION_COUNT , STABLE ) ;
    exit when STABLE ;
  end loop ;
--
  ITERATION.FEATURE_TIMES ( CPU_TIME , WALL_TIME ) ;
  CPU_TIME := DURATION ( CPU_TIME / CASE_COUNT ) ;
  WALL_TIME := DURATION ( WALL_TIME / CASE_COUNT ) ;

--
-- Printout
--
  PIWG_IO.PIWG_OUTPUT ( "T000004" , "Tasking" ,
                        CPU_TIME , WALL_TIME , ITERATION_COUNT ,
     " Task entry call and return time measured" ,
     " One tasks active, two entries, tasks in a package " ,
     " using select statement " ) ;

  abort T1 ;
end T000004 ;

package body TASK_PACK_4 is
  task body T1 is
  begin
    loop
      select
        accept E1 do
          GLOBAL := GLOBAL + A_ONE ;
          REMOTE ;
        end E1 ;
      or
        accept E2 do
          GLOBAL := GLOBAL + A_ONE ;
          REMOTE ;
        end E2 ;
      end select ;
    end loop ;
  end T1 ;

end TASK_PACK_4 ;
