with PHYSICAL_REAL ; use PHYSICAL_REAL ;

package PHYSICAL_UNITS_MECHANICAL is

-- This package specification defines Ada types for physical
-- quantities generally in the mechanical context.
--
-- This package is the logical continuation of PHYSICAL_UNITS_BASIC
--
--
--                                 DERIVED MECHANICAL UNITS
--
--
--                                    2                              2
--area                       A       L           square meter       m
--
  type AREA_MKS is new REAL ;
  subtype AREA_SQUARE_METER is AREA_MKS ;
  subtype AREA_SQUARE_METERS is AREA_MKS ;
  type AREA_ENGLISH is new REAL ;
  subtype AREA_SQUARE_FEET is AREA_ENGLISH ;
  subtype AREA_SQUARE_FOOT is AREA_ENGLISH ;
  type AREA_SQUARE_CENTIMETER is new REAL ;
  type AREA_SQUARE_KILOMETER is new REAL ;
  type AREA_SQUARE_INCH is new REAL ;
  type AREA_SQUARE_YARD is new REAL ;
  type AREA_SQUARE_MILE is new REAL ;
  type AREA_ACRE is new REAL ;
  type AREA_CIRCULAR_MIL is new REAL ;
  type AREA_HECTARE is new REAL ;
  type AREA_TOWNSHIP is new REAL ;

--
--                                    3                              3
--volume                     V       L           stere              m
--
  type VOLUME_MKS is new REAL ;
  subtype VOLUME_STERE is VOLUME_MKS ;
  subtype VOLUME_CUBIC_METER is VOLUME_MKS ;
  type VOLUME_ENGLISH is new REAL ;
  subtype VOLUME_CUBIC_FEET is VOLUME_ENGLISH ;
  type VOLUME_MILLILITER is new REAL ;
  type VOLUME_LITER is new REAL ;
  type VOLUME_KILOLITER is new REAL ;
  type VOLUME_CUBIC_CENTIMETER is new REAL ;
  type VOLUME_CUBIC_INCH is new REAL ;
  type VOLUME_CUBIC_YARD is new REAL ;
  type VOLUME_CUBIC_MILE is new REAL ;
  type VOLUME_TEASPOON is new REAL ;
  type VOLUME_TABLESPOON is new REAL ;
  type VOLUME_OUNCE_FLUID is new REAL ;
  type VOLUME_JIGGER is new REAL ;
  type VOLUME_CUP is new REAL ;
  type VOLUME_PINT_LIQUID is new REAL ;
  type VOLUME_QUART_LIQUID is new REAL ;
  type VOLUME_GALLON is new REAL ;
  type VOLUME_KEG is new REAL ;
  type VOLUME_BARREL is new REAL ;
  type VOLUME_PINT_DRY is new REAL ;
  type VOLUME_QUART_DRY is new REAL ;
  type VOLUME_PECK is new REAL ;
  type VOLUME_BUSHEL is new REAL ;
  type VOLUME_CORD is new REAL ;

--
--velocity                   v       L/T         meter per second   m/sec
--
  type VELOCITY_MKS is new REAL ;
  subtype VELOCITY_METER_PER_SECOND is VELOCITY_MKS ;
  type VELOCITY_ENGLISH is new REAL ;
  subtype VELOCITY_FEET_PER_SECOND is VELOCITY_ENGLISH ;
  type VELOCITY_CENTIMETER_PER_SECOND is new REAL ;
  type VELOCITY_KILOMETER_PER_HOUR is new REAL ;
  type VELOCITY_INCHES_PER_SECOND is new REAL ;
  type VELOCITY_MILE_PER_HOUR is new REAL ;
  type VELOCITY_MILES_PER_SECOND is new REAL ;
  type VELOCITY_INCHES_PER_MINUTE is new REAL ;
  type VELOCITY_FEET_PER_MINUTE is new REAL ;
  type VELOCITY_MILES_PER_HOUR is new REAL ;
  type VELOCITY_KNOTS is new REAL ;
  type VELOCITY_FURLONG_PER_FORTNIGHT is new REAL ;

--
--angular velocity           omega   1/T         radians per second 1/sec
--
  type ANGULAR_VELOCITY is new REAL ;
  subtype ANGULAR_VELOCITY_RADIANS_PER_SECOND is ANGULAR_VELOCITY ;
  type ANGULAR_VELOCITY_DEGREES_PER_SECOND is new REAL ;
  type ANGULAR_VELOCITY_REVOLUTIONS_PER_MINUTE is new REAL ;
  type ANGULAR_VELOCITY_REVOLUTIONS_PER_SECOND is new REAL ;

--
--                                      2                                2
--acceleration               a       L/T         meter per second   m/sec
--                                               squared
--
  type ACCELERATION_MKS is new REAL ;
  subtype ACCELERATION_METER_PER_SECOND_SQUARED is ACCELERATION_MKS ;
  type ACCELERATION_ENGLISH is new REAL ;
  subtype ACCELERATION_FEET_PER_SECOND_SQUARED is ACCELERATION_ENGLISH ;

--
--                                      2                                2
--angular acceleration       alpha   1/T         radians per        1/sec
--                                               square second
--
  type ANGULAR_ACCELERATION is new REAL ;
  subtype ANGULAR_ACCELERATION_RADIANS_PER_SECOND_SQUARED is 
                 ANGULAR_ACCELERATION ;
  type ANGULAR_ACCELERATION_REVOLUTIONS_PER_MINUTE_SQUARED is new REAL ;

--
--                                       2                                  2
--force                      F       ML/T        newton             Kg m/sec
--
  type FORCE_MKS is new REAL ;
  subtype FORCE_NEWTON is FORCE_MKS ;
  subtype FORCE_KILOGRAM_METER_PER_SECOND_SQUARED is FORCE_MKS ;
  type FORCE_DYNE is new REAL ;
  type FORCE_ENGLISH is new REAL ;
  subtype FORCE_POUNDAL is FORCE_ENGLISH ;
  subtype FORCE_POUND_FOOT_PER_PER_SECOND_SQUARED is FORCE_ENGLISH ;

--
--                                     2  2                             2    2
--energy                     E       ML /T       joule              Kg m /sec
--work                       W         "           "                   "
--heat                       Q         "           "                   "
--torque (moment)            T         "         newton meter          "
--
  type ENERGY_MKS is new REAL ;
  subtype WORK_MKS is ENERGY_MKS ;
  subtype HEAT_MKS is ENERGY_MKS ;
  subtype TORQUE_MKS is ENERGY_MKS ;
  subtype ENERGY_JOULE is ENERGY_MKS ;
  subtype ENERGY_NEWTON_METER is ENERGY_MKS ;
  subtype ENERGY_KILOGRAM_METER_SQUARED_PER_SECOND_SQUARED is ENERGY_MKS ;
  type ENERGY_ERG is new REAL ;
  type ENERGY_GRAM_CALORIE is new REAL ;
  type ENERGY_KILOGRAM_CALORIE is new REAL ;
  type ENERGY_ENGLISH is new REAL ;
  subtype ENERGY_B_T_U is ENERGY_ENGLISH ;
  type ENERGY_FOOT_POUND is new REAL ;
  type ENERGY_KILOWATT_HOUR is new REAL ;
  type ENERGY_HORSEPOWER_HOUR is new REAL ;

--
--                                     2  3                        
--power                      P       ML /T       watt               joule/sec
--
  type POWER_MKS is new REAL ;
  subtype POWER_WATT is POWER_MKS ;
  subtype POWER_JOULE_PER_SECOND is POWER_MKS ;
  subtype POWER_VOLT_AMPERE is POWER_MKS ;
  type POWER_KILOGRAM_CALORIE_PER_SECOND is new REAL ;
  type POWER_KILOGRAN_CALORIE_PER_MINUTE is new REAL ;
  type POWER_HORSEPOWER_MECHANICAL is new REAL ;
  type POWER_HORSEPOWER_ELECTRICAL is new REAL ;
  type POWER_HORSEPOWER_METRIC is new REAL ;
  type POWER_HORSEPOWER_BOILER is new REAL ;
  type POWER_B_T_U_PER_MINUTE is new REAL ;
  type POWER_B_T_U_PER_HOUR is new REAL ;
  type POWER_FOOT_POUND_PER_MINUTE is new REAL ;
  type POWER_FOOT_POUND_PER_SECOND is new REAL ;

--
--                                      3                               3
--density                    D       M/L         kilogram per       Kg/m
--                                               cubic meter
--
  type DENSITY_MKS is new REAL ;
  subtype DENSITY_KILOGRAM_PER_CUBIC_METER is DENSITY_MKS ;
  type DENSITY_ENGLISH is new REAL ;
  subtype DENSITY_POUND_PER_CUBIC_FOOT is DENSITY_ENGLISH ;

--
--                                    3                              3
--flow rate                  f       L /T        cubic meter per    m /sec
--                                               second
--
  type FLOW_RATE_MKS is new REAL ;
  subtype FLOW_RATE_CUBIC_METER_PER_SECOND is FLOW_RATE_MKS ;
  type FLOW_RATE_ENGLISH is new REAL ;
  subtype FLOW_RATE_CUBIC_FEET_PER_SECOND is FLOW_RATE_ENGLISH ;
  type FLOW_RATE_GALLON_PER_MINUTE is new REAL ;
  type FLOW_RATE_CUBIC_FEET_PER_MINUTE is new REAL ;

--
--                                       2                                  2
--pressure                   P       M/LT        pascal             Kg/m sec
-- stress                                        newton per
-- energy density                                square meter
--
  type PRESSURE_MKS is new REAL ;
  subtype PRESSURE_PASCAL is PRESSURE_MKS ;
  subtype PRESSURE_NEWTON_PER_SQUARE_METER is PRESSURE_MKS ;
  subtype PRESSURE_FORCE_PER_AREA_MKS is PRESSURE_MKS ;
  subtype PRESSURE_JOULE_PER_CUBIC_METER is PRESSURE_MKS ;
  subtype PRESSURE_ENERGY_DENSITY_MKS is PRESSURE_MKS ;
  type PRESSURE_ENGLISH is new REAL ;
  subtype PRESSURE_POUND_PER_SQUARE_FOOT is PRESSURE_ENGLISH ;
  type PRESSURE_TON_PER_SQUARE_FOOT is new REAL ;
  type PRESSURE_ATMOSPHERE_STANDARD is new REAL ;
  type PRESSURE_FEET_OF_WATER is new REAL ;
  type PRESSURE_INCHES_OF_MERCURY is new REAL ;
  type PRESSURE_MILLIMETER_OF_MERCURY is new REAL ;
  type PRESSURE_BAR is new REAL ;
  type PRESSURE_MILLIBAR is new REAL ;
  type PRESSURE_TORR is new REAL ;

--
--momentum                   p       ML/T        newton second       Kg m/sec
--
  type MOMENTUM_MKS is new REAL ;
  subtype MOMENTUM_NEWTON_SECOND is MOMENTUM_MKS ;
  subtype MOMENTUM_KILOGRAM_METER_PER_SECOND is MOMENTUM_MKS ;

--
--                                     2                                2
--inertia                    I       ML /T       joule second       Kg m /sec
--
  type INERTIA_MKS is new REAL ;
  subtype INERTIA_JOULE_SECOND is INERTIA_MKS ;
  subtype INERTIA_KILOGRAM_METER_SQUARED_PER_SECOND is INERTIA_MKS ;

--
--                                     2                                2
--moment of inertia          M       ML          kilogram           Kg m 
--                                               meter squared
--
  type MOMENT_OF_INERTIA_MKS is new REAL ;
  subtype MOMENT_OF_INERTIA_KILOGRAM_METER_SQUARED is MOMENT_OF_INERTIA_MKS ;

--
--                                    2                                2
--kinematic viscosity        v       M /T        kilogram squared    Kg /sec 
--                                               per second
--
  type KINEMATIC_VISCOSITY_MKS is new REAL ;
  subtype KINEMATIC_VISCOSITY_KILOGRAM_SQUARED_PER_SECOND is 
          KINEMATIC_VISCOSITY_MKS ;
--
--dynamic viscosity          d       M/LT        newton second       Kg/m sec 
--                                               per square meter
--
  type DYNAMIC_VISCOSITY_MKS is new REAL ;
  subtype DYNAMIC_VISCOSITY_NEWTON_PER_SQUARE_METER is DYNAMIC_VISCOSITY_MKS ;
  subtype DYNAMIC_VISCOSITY_KILOGRAM_PER_METER_SECOND is DYNAMIC_VISCOSITY_MKS ;
--
--
--luminous flux              phi     C           lumen (4Pi candle   cd sr 
--                                               for point source)
--
  type LUMINOUS_FLUX_LUMEN is new REAL ;

--
--                                      2                                   2
--illumination               E       C/L         lumen per           cd sr/m
--                                               square meter
--
  type ILLUMINATION_MKS is new REAL ;
  subtype ILLUMINATION_LUMEN_PER_SQUARE_METER is ILLUMINATION_MKS ;

--
--                                      2                                 2
--luminance                  l       C/L         lux                  cd/m
--                                               square meter
--
  type LUMINANCE_MKS is new REAL ;
  subtype LUMINANCE_LUX is LUMINANCE_MKS ;
  subtype LUMINANCE_CANDLE_PER_SQUARE_METER is LUMINANCE_MKS ;

--
--
--                                     2  2                             2 
--entropy                    S       ML /T K     joule per degree   Kg m /
--                                                                      2 o
--                                                                   sec   K
--
  type ENTROPY_MKS is new REAL ;
  subtype SPECIFIC_HEAT_MKS is ENTROPY_MKS ;
  subtype SPECIFIC_HEAT_JOULE_PER_DEGREE_KELVIN is ENTROPY_MKS ;
  type SPECIFIC_HEAT_B_T_U_PER_POUND_DEGREE_FARENHEIT is new REAL ;

--
end PHYSICAL_UNITS_MECHANICAL ;
