with PHYSICAL_REAL ; use PHYSICAL_REAL ;

package PHYSICAL_UNITS_ELECTRICAL is

-- This package specification defines Ada types for physical
-- quantities related to electrical units. It ia a logical
-- extension of PHYSICAL_UNITS_MECHANICAL .
--
--
--
--                                  DERIVED ELECTRICAL
--
--electric current           I       Q/T         ampere             c/sec
--  magnetomotive force

  type CURRENT_AMPERE is new REAL ;
  type CURRENT_MILLIAMPERE is new REAL ;
  type CURRENT_MICROAMPERE is new REAL ;
  type CURRENT_ABAMPERE is new REAL ;
  type CURRENT_STATAMPERE is new REAL ;

--
--                                     2  2                             2    2
--voltage                    E       ML /T Q     volt               Kg m /sec c
--  potential difference
--  electromotive force

  type VOLTAGE_VOLT is new REAL ;
  type VOLTAGE_MILLIVOLT is new REAL ;
  type VOLTAGE_MICROVOLT is new REAL ;
  type VOLTAGE_KILOVOLT is new REAL ;

--
--                                     2   2                            2      2
--electric resistance        R       ML /TQ      ohm                Kg m /sec c
--
  type RESISTANCE_OHM is new REAL ;
  type RESISTANCE_MILLIOHM is new REAL ;
  type RESISTANCE_KILOHM is new REAL ;
  type RESISTANCE_MEGOHM is new REAL ;

--
--                                     3   2                            3      2
--electric resistivity       rho     ML /TQ     ohm meter           Kg m /sec c
--
  type RESISTIVITY_OHM_METER is new REAL ;

--
--                                     2   2                             2     2
--electric conductance       G       TQ /ML      mho                sec c /Kg m
--
  type CONDUCTANCE_MHO is new REAL ;

--
--                                     2   3                             2     3
--conductivity               sigma   TQ /ML      mho per meter      sec c /Kg m
--
  type CONDUCTIVITY_MHO_PER_METER is new REAL ;

--
--
--                                    2 2   2                          2 2     2
--capacitance                C       T Q /ML     farad              sec c /Kg m
--
  type CAPACITANCE_FARAD is new REAL ;
  type CAPACITANCE_MICROFARAD is new REAL ;
  type CAPACITANCE_PICOFARAD is new REAL ;

--
--
--                                     2  2                             2  2
--inductance                 L       ML /Q       henry              Kg m /c
--                                               weber per ampere
--                                               volt second per ampere

--
  type INDUCTANCE_HENRY is new REAL ;
  type INDUCTANCE_MILLIHENRY is new REAL ;
  type INDUCTANCE_MICROHENRY is new REAL ;

--
--                                       2                                 2
--current density            J       Q/TL        ampere per         c/sec m
--                                               square meter
--
  type CURRENT_DENSITY_AMPERE_PER_SQUARE_METER is new REAL ;

--
--                                      3                              3
--charge density             rho     Q/L         coulomb per        c/m
--                                               cubic meter
--
  type CHARGE_DENSITY_COULOMB_PER_CUBIC_METER is new REAL ;

--                                     2                                2
--magnetic flux              F       ML /TQ      weber              Kq m /sec c
--                                               volt second
--
  type MAGNETIC_FLUX_WEBER is new REAL ;

--
--magnetic flux density,     B       M/TQ        tesla              Kq/sec c
--   magnetic induction                          weber per square meter
--
  type MAGNETIC_FLUX_DENSITY is new REAL ;
  subtype MAGNETIC_FLUX_DENSITY_TESLA is MAGNETIC_FLUX_DENSITY ;
  subtype MAGNETIC_FLUX_DENSITY_WEBER_PER_SQUARE_METER is
          MAGNETIC_FLUX_DENSITY ;

--
--magnetic intensity         H       Q/LT        ampere per meter   c/m sec
--  magnetic field strength
--
  type MAGNETIC_INTENSITY is new REAL ;
  subtype MAGNETIC_INTENSITY_AMPERE_PER_METER is MAGNETIC_INTENSITY ;

--
--
--magnetic vector potential  A       ML/TQ       weber/meter        Kg m/sec c
--
  type MAGNETIC_VECTOR_POTENTIAL_WEBER_PER_METER is new REAL ;

--
--                                       2                                  2
--electric field intensity   E       ML/T Q      volt/meter         Kg m/sec c
--  electric field strength                      newton per coulomb
--
  type ELECTRIC_FIELD is new REAL ;
  subtype ELECTRIC_FIELD_INTENSITY_VOLT_PER_METER is 
          ELECTRIC_FIELD ;

--
--                                      2                              2
--electric displacement      D       Q/L         coulomb per        c/m
--                                               square meter
--
  type ELECTRIC_DISPLACEMENT is new REAL ;
  subtype ELECTRIC_DISPLACEMENT_COULOMB_PER_SQUARE_METER is 
          ELECTRIC_DISPLACEMENT ;

--
--                                       2                                2
--permeability               mu      ML/Q        henry per meter    Kg m/c
--
  type PERMEABILITY is new REAL ;
  subtype PERMEABILITY_HENRY_PER_METER is PERMEABILITY ;

--
--                                    2 2   3                          2 2     3
--permittivity,              epsi    T Q /ML     farad per meter    sec c /Kg m
--  dielectric constant
--
  type PERMITTIVITY is new REAL ;
  subtype PERMITTIVITY_FARAD_PER_METER is PERMITTIVITY ;
  subtype DIELECTRIC_CONSTANT is PERMITTIVITY ;

--
--                                                                     -1
--frequency                  f       Pi/T        hertz              sec
--
  type FREQUENCY_HERTZ is new REAL ;
  type FREQUENCY_KILOHERTZ is new REAL ;
  type FREQUENCY_MEGAHERTZ is new REAL ;
  type FREQUENCY_GIGAHERTZ is new REAL ;

--
--                                                                     -1
--angular frequency          omega   1/T         radians per second sec
--
  type ANGULAR_FREQUENCY_RADIAN_PER_SECOND is new REAL ;

--
end PHYSICAL_UNITS_ELECTRICAL ;
