with PHYSICAL_REAL ; use PHYSICAL_REAL ;

package PHYSICAL_UNITS_OTHER is

-- This package specification defines Ada types for physical
-- units that occur as intermediate results.
-- A number of other packages use this package.
--
--
--   The comments below are organized to present the physical quantity unit with
--associated information. The first column is the dimension of the physical
--quantity expressed in terms of the fundamental dimensions. The second column
--is the typical MKS unit equation. 
--
-- DIMENSION   UNIT EQUATION
-- _________   _____________
--
--                TYPES NEEDED FOR COMPUTATIONS
--
--    2            2
--   T          sec
  type TIME_SECOND_SQUARED is new REAL ;

--
--   2  2        2    2
--  L /T        m /sec
  type VELOCITY_SQUARED_MKS is new REAL ;
  subtype VELOCITY_MKS_SQUARED is VELOCITY_SQUARED_MKS ;

--    2  2            o
--  ML /T K     joule/ K 
  type JOULE_PER_DEGREE_KELVIN is new REAL ;

--
--    3  2 2
--  ML /T Q     m/farad
  type METER_PER_FARAD is new REAL ;

--
--   2 4  4 2       2
--  M L /T Q    volt
  type VOLT_SQUARED  is new REAL ;

--
--   2  2             2
--  Q /T        ampere
  type AMPERE_SQUARED is new REAL ;

--
--     2
--  Q/T         ampere/sec
  type AMPERE_PER_SECOND is new REAL ;

-- 
--    2  3
--  ML /T Q     volt/sec
  type VOLT_PER_SECOND is new REAL ;

--
--   2   2
--  L /MT
  type ACCELERATION_PER_KILOGRAM is new REAL ;

--
end PHYSICAL_UNITS_OTHER ;
