
package PHYSICAL_CONVERSION_CONSTANT is

-- short names for constants used to derive other constants
  C : constant := 2.997925E8 ;
  PI : constant := 3.14159 ;

--
-- length
--
  FOOT_TO_METER : constant := 0.3048 ; -- exact 
  METER_TO_FOOT : constant := 1.0 / FOOT_TO_METER ;
  FOOT_TO_MIL : constant := 12.0 * 1000.0 ;
  FOOT_TO_INCH : constant := 12.0 ;
  FOOT_TO_YARD : constant := 1.0 / 3.0 ;
  FOOT_TO_FATHOM : constant := 1.0 / 6.0 ;
  FOOT_TO_CABLES_LENGTH : constant := FOOT_TO_FATHOM / 100.0 ;
  FOOT_TO_CABLES_LENGTH_U_S_NAVY : constant := FOOT_TO_FATHOM / 120.0 ;
  FOOT_TO_ROD : constant := 1.0 / 16.5 ;  -- also POLE or PERCH
  FOOT_TO_LINK_SURVEYOR : constant := 100.0 / 66.0 ;  -- 1/100 chain
  FOOT_TO_CHAIN_SURVEYOR : constant := 1.0 / 66.0 ;  -- or Gunter's 1/80 mile
  FOOT_TO_LINK_ENGINEER : constant := 1.0 ;
  FOOT_TO_CHAIN_ENGINEER : constant := 1.0 / 100.0 ;
  FOOT_TO_FURLONG : constant := 1.0 / 660.0 ;  -- 40 rods or 1/8 mile
  FOOT_TO_MILE_STATUTE : constant := 1.0 / 5280 ;
  FOOT_TO_MILE_NAUTICAL : constant := 1.0 / 6000.0 ;  -- 10 CABLES_LENGTH

--                      length of a minute of longitude at the equator
--                      exactly 1.852 kilometers
  FOOT_TO_MILE_NAUTICAL_INTERNATIONAL : constant := 1.0 / 6076.11549 ;
  FOOT_TO_LEAGUE_LAND : constant := FOOT_TO_MILE_STATUTE / 3.0 ;
  FOOT_TO_LEAGUE_MARINE : constant := FOOT_TO_MILE_NAUTICAL / 3.0 ;
  FOOT_TO_LIGHT_YEAR : constant := 1.0 / 3.103647E16 ;
  MILE_TO_FOOT : constant := 5_280.0 ;
  METER_TO_PICOMETER : constant := 1.0E12 ;
  METER_TO_ANGSTROM : constant := 1.0E10 ;
  METER_TO_NANOMETER : constant := 1.0E9 ;
  METER_TO_MICROMETER : constant := 1.0E6 ;
  METER_TO_MILLIMETER : constant := 1000.0 ;
  METER_TO_CENTIMETER : constant := 100.0 ;
  METER_TO_DECIMETER : constant := 10.0 ;
  METER_TO_DECAMETER : constant := 0.1 ;
  METER_TO_HECTOMETER : constant := 0.01 ;
  METER_TO_KILOMETER : constant := 0.001 ;
  METER_TO_MYRIAMETER : constant := 0.0001 ;
  METER_TO_MEGAMETER : constant := 1.0E-6 ;
  METER_TO_GIGAMETER : constant := 1.0E-9 ;

--
-- mass
--
  KILOGRAM_TO_POUND : constant := 2.204622341 ;  -- avdp Avoirdupois
  KILOGRAM_TO_GRAIN : constant := 1.0 / 0.0648E-3 ;  -- all systems
  KILOGRAM_TO_CARAT : constant := 5000.0 ;  -- exact
  POUND_TO_KILOGRAM : constant := 1.0 / KILOGRAM_TO_POUND ;
  POUND_TO_POUND_TROY : constant := 7000.0 / 5760.0 ;  -- Troy or Apothecary
  POUND_TO_HUNDREDWEIGHT : constant := 0.01 ;  -- avdp
  POUND_TO_STONE : constant := 112.0 ;
  POUND_TO_TON_SHORT : constant := 1.0 / 2000.0 ;  -- avdp
  POUND_TO_TON_LONG : constant := 1.0 / 2240.0 ;  -- avdp
  DRAM_TO_GRAM : constant := 1.772 ;
  POUND_TO_OUNCE : constant := 16.0 ;  -- avdp
  POUND_TO_DRAM : constant := 256.0 ;  -- avdp , ounce is 16 drams
  POUND_TO_GRAIN : constant := 7000.0 ;  -- avdp
  POUND_TROY_TO_GRAIN : constant := 5700.0 ;  -- CHECK. A GRAIN IS A GRAIN
  POUND_TROY_TO_PENNYWEIGHT : constant := 240.0 ;  -- troy
  POUND_TROY_TO_CARAT : constant := 5760.0 / 3.086 ;  -- troy (exact metric)
  POUND_TROY_TO_OUNCE_TROY : constant := 12.0 ;
  POUND_TROY_TO_DRAM : constant := 8.0 * 16.0 ;  -- apoth
  POUND_TROY_TO_SCRUPLE : constant := 24.0 * 12.0 ;  -- apoth
  KILOGRAM_TO_MILLIGRAM : constant := 1.0E6 ;
  KILOGRAM_TO_GRAM : constant := 1000.0 ;
  KILOGRAM_TO_TON_METRIC : constant := 0.001 ;

--
-- time
--
  SECOND_TO_PICOSECOND : constant := 1.0E12 ;
  SECOND_TO_NANOSECOND : constant := 1.0E9 ;
  SECOND_TO_MICROSECOND : constant := 1.0E6 ;
  SECOND_TO_MILLISECOND : constant := 1000.0 ;
  SECOND_TO_CENTISECOND : constant := 100.0 ;
  SECOND_TO_KILOSECOND : constant := 1.0E-3 ;
  SECOND_TO_MEGASECOND : constant := 1.0E-6 ;
  SECOND_TO_GIGASECOND : constant := 1.0E-9 ;
  SECOND_TO_MINUTE : constant := 1.0 / 60.0 ;
  SECOND_TO_HOUR : constant := SECOND_TO_MINUTE / 60.0 ;
  SECOND_TO_DAY : constant := SECOND_TO_HOUR / 24.0 ;
  SECOND_TO_FORTNIGHT : constant := SECOND_TO_DAY / 14.0 ;
  SECOND_TO_YEAR : constant := SECOND_TO_DAY / 365.241 ;
  SECOND_TO_MONTH : constant := SECOND_TO_YEAR * 12.0 ;
  SECOND_TO_DECADE : constant := SECOND_TO_YEAR / 10.0 ;
  SECOND_TO_CENTURY : constant := SECOND_TO_YEAR / 100.0 ;
  SECOND_TO_MILLENNIA : constant := SECOND_TO_YEAR / 1000.0 ;

--
-- electric charge, electric flux
--
  COULOMB_TO_AMPERE_SECOND : constant := 1.0 ;  -- definition
  COULOMB_TO_ABCOULOMB : constant := 0.1 ;  -- cgs emu
  COULOMB_TO_STSTCOULOMB : constant := 10.0 * C ;  -- cgs esu
  AMPERE_SECOND_TO_AMPERE_HOURS : constant := 1.0 / 3600.0 ;
  AMPERE_SECOND_TO_ELECTRON : constant := 1.0 / 1.60193E-19 ;
  AMPERE_SECOND_TO_FARADAY : constant := 1.0365E-5 ;

--
-- luminous intensity, the flux emitted by 1 candel is 4 Pi lumens
--
  CANDLE_TO_CANDEL : constant := 1.0 ;  -- see LUX, LUMEN, PHOT

--
-- temperature ( built into equations )
--
  CENTIGRADE_TO_CELSIOUS : constant := 1.0 ;

--
-- angle
--
  DEGREE_TO_RADIAN : constant := 0.0174533 ;
  RADIAN_TO_DEGREE : constant := 57.295780 ;
  RADIAN_TO_MINUTE : constant := RADIAN_TO_DEGREE * 60.0 ;
  RADIAN_TO_SECOND : constant := RADIAN_TO_MINUTE * 60.0 ;
  RADIAN_TO_REVOLUTION : constant := 1.0 / ( 2.0 * PI ) ;
  RADIAN_TO_BAM : constant := 1.0 / PI ;

--
-- solid angle
--
  STERADIAN_TO_AREA_OF_SPHERE : constant := 1.0 / ( 4.0 * PI ) ;

--
-- area
--
  SQUARE_METER_TO_SQUARE_FOOT : constant := METER_TO_FOOT * METER_TO_FOOT ;
  SQUARE_FOOT_TO_SQUARE_METER : constant := FOOT_TO_METER * FOOT_TO_METER ;

--
-- volume
--
  CUBIC_METER_TO_CUBIC_FOOT : constant := SQUARE_METER_TO_SQUARE_FOOT *
      METER_TO_FOOT ;
  CUBIC_FOOT_TO_CUBIC_METER : constant := SQUARE_FOOT_TO_SQUARE_METER *
      FOOT_TO_METER ;

--
-- velocity
--
  METER_PER_SECOND_TO_FOOT_PER_SECOND : constant := METER_TO_FOOT ;

--
-- angular velocity
--
  RADIAN_PER_SECOND_TO_DEGREE_PER_SECOND : constant := RADIAN_TO_DEGREE ;

--
-- acceleration
--
  METER_PER_SECOND_SQUARED_TO_FOOT_PER_SECOND_SQUARED : constant :=
      METER_TO_FOOT ;

--
-- angular acceleration
--
  RADIAN_PER_SECOND_SQUARED_TO_DEGREE_PER_SECOND_SQUARED : constant :=
      RADIAN_TO_DEGREE ;

--
-- force
--
  POUNDS_TO_POUNDALS : constant := 32.1725 ;
  NEWTON_TO_DYNE : constant := 1.0E5 ;  -- cgs
  NEWTON_TO_POUND : constant := KILOGRAM_TO_POUND * METER_TO_FOOT ;
  NEWTON_TO_POUNDAL : constant := 7.23300 ;

--
-- energy, work, heat, torque ( moment )
--
  JOULE_TO_ERG : constant := 1.0E7 ;  -- cgs

--
-- power
--
  WATT_TO_ERG_PER_SECOND : constant := 1.0E7 ;  -- cgs

--
-- density
--
  KILOGRAM_PER_CUBIC_METER_TO_POUND_PER_CUBIC_FOOT : constant :=
      KILOGRAM_TO_POUND / CUBIC_METER_TO_CUBIC_FOOT ;

--
-- flow rate
--
  CUBIC_METER_PER_SECOND_TO_CUBIC_FOOT_PER_SECOND : constant :=
      CUBIC_METER_TO_CUBIC_FOOT ;

--
-- pressure, stress, energy density
--
  PASCAL_TO_NEWTON_PER_SQUARE_METER : constant := 1.0 ;
  PASCAL_TO_POUND_PER_SQUARE_FOOT : constant := NEWTON_TO_POUND /
      SQUARE_METER_TO_SQUARE_FOOT ;

--
-- momentum
--
  NEWTON_SECOND_TO_POUND_SECOND : constant := NEWTON_TO_POUND ;

--
-- inertia
--
  JOULE_SECOND_TO_POUND_SQUARE_FOOT_PER_SECOND : constant := KILOGRAM_TO_POUND
      * SQUARE_METER_TO_SQUARE_FOOT ;

--
-- moment of inertia
--
  KILOGRAM_SQUARE_METER_TO_POUND_SQUARE_FOOT : constant := KILOGRAM_TO_POUND *
      SQUARE_METER_TO_SQUARE_FOOT ;

--
-- kinematic viscosity
--
  KILOGRAM_SQUARED_PER_SECOND_TO_POUND_SQUARED_PER_SECOND : constant :=
      KILOGRAM_TO_POUND * KILOGRAM_TO_POUND ;

--
-- viscosity, dynamic viscosity
--
  KILOGRAM_PER_METER_SECOND_TO_POUND_PER_FOOT_SECOND : constant :=
      KILOGRAM_TO_POUND / METER_TO_FOOT ;

--
-- luminous flux
--
  LUMEN_TO_LUMEN : constant := 1.0 ;

--
-- illumination
--
  LUMEN_PER_SQUARE_METER_TO_LUX : constant := 1.0 ;  -- metre candle
  LUMEN_PER_SQUARE_METER_TO_FOOT_CANDLE : constant := 10.76 ;  -- lumen/ sq ft
  LUMEN_PER_SQUARE_METER_TO_PHOT : constant := 1.0E4 ;  -- cm candle

--
-- luminance
--
  CANDEL_PER_SQUARE_METER_TO_LUX : constant := 1.0 ;

--
-- entropy
--
  JOULE_PER_DEGREE_KELVIN_TO_B_T_U_PER_POUND_DEGREE_FARENHEIT : constant :=
      999999999.99999999999 ;  -- ?

--
-- electric current, magnetomotive force, I
--
  AMPERE_TO_ABAMPERE : constant := 0.1 ;  -- cgs emu
  AMPERE_TO_STATAMPERE : constant := 10.0 * C ;  -- cgs esu
  AMPERE_TO_GILBERT : constant := 4.0 * PI / 10.0 ;
  AMPERE_TO_AMPERE_TURN : constant := 1.0 ;

--
-- voltage, potential difference, electromotive force ( emf ), E
--
  VOLT_TO_ABVOLT : constant := 1.0E8 ;  -- cgs emu
  VOLT_TO_STSTVOLT : constant := 1.0E6 * C ;

--
-- electric resistance, R
--
  OHM_TO_ABOHM : constant := 1.0E9 ;  -- cgs emu
  OHM_TO_STATOHM : constant := 1.0E5 * C * C ;  -- cgs esu

--
-- electric resistivity
--
  OHM_METER_TO_ABOHM_CENTIMETER : constant := 1.0E11 ;

--
-- electric conductance, G
--
  MHO_TO_ABMHO : constant := 1.0E-9 ;  -- cgs emu

--
-- electric conductivity
--
  MHO_PER_METER_TO_ABMHO_PER_CENTIMETER : constant := 1.0E-11 ;  -- cgs emu

--
-- capacitance, C
--
  FARAD_TO_ABFARAD : constant := 1.0E-9 ;  -- cgs emu
  FARAD_TO_STATFARAD : constant := 1.0E-5 * C * C ;  -- cgs esu
  FARAD_TO_MICROFARAD : constant := 1.0E6 ;
  FARAD_TO_PICOFARAD : constant := 1.0E12 ;

--
-- inductance, L
--
  HENRY_TO_ABHENRY : constant := 1.0E9 ;  -- cgs emu
  HENRY_TO_STATHENRY : constant := 1.0E5 / ( C * C ) ;  -- cgs esu
  HENRY_TO_MILLIHENRY : constant := 1000.0 ;
  HENRY_TO_MICROHENRY : constant := 1.0E6 ;

--
-- current density, J
--
  AMPERE_PER_SQUARE_METER : constant := 1.0 ;

--
-- charge density
--
  COULOMB_PER_CUBIC_METER : constant := 1.0 ;

--
-- magnetic flux, magnetic pole
--
  WEBER_TO_MAXWELL : constant := 1.0E8 ;  -- cgs
  WEBER_TO_VOLT_SECOND : constant := 1.0 ;
  WEBER_TO_POLE : constant := 1.0E8 / ( 4.0 * PI ) ;  -- cgs

--
-- magnetic flux density, magnetic induction, B
--
  TESLA_TO_WEBER_PER_SQUARE_METER : constant := 1.0 ;
  TESLA_TO_GAUSS : constant := 1.0E4 ;  -- cgs

--
-- magnetic intensity, magnetic field strength, H
--
  AMPERE_PER_METER_TO_OERSTED : constant := 0.01 * 4.0 * PI ;
  AMPERE_PER_METER_TO_NEWTON_PER_WEBER : constant := 1.0 ;

--
-- magnetic vector potential, A
--
  WEBER_PER_METER : constant := 1.0 ;

--
-- electric fiels strength, electric field intensity, E
--
  VOLT_PER_METER_TO_NEWTON_PER_COULOMB : constant := 1.0 ;

--
-- electric displacement, D
--
  ELECTRIC_DISPLACEMENT_TO_COULOMB_PER_SQUARE_METER : constant := 1.0 ;

--
-- permeability
--
  PERMEABILITY_TO_HENRY_PER_METER : constant := 1.0 ;

--
-- permittivity, dielectric constant
--
  PERMITTIVITY_TO_FARAD_PER_METER : constant := 1.0 ;

--
-- frequency
--
  HERTZ_TO_KILOHERTZ : constant := 1.0E-3 ;
  HERTZ_TO_MEGAHERTZ : constant := 1.0E-6 ;
  HERTZ_TO_GIGAHERTZ : constant := 1.0E-9 ;
  HERTZ_TO_RADIAN_PER_SECOND : constant := 1.0 / ( 2.0 * PI ) ;

--
-- angular frequency, omega = 2 Pi f
--
  RADIAN_PER_SECOND_TO_HERTZ : constant := 2.0 * PI ;

--
end PHYSICAL_CONVERSION_CONSTANT ;
