
-- The combinational problem of converting from every unit to every
-- other units of the same dimension is too large to be practical.
--
-- The compromise solution is to provide:
--   MKS,     conversion of every metric unit to the corresponding MKS unit 
--   MKS,     conversion of primary English unit to corresponding MKS unit
--   ENGLISH, conversion of every English unit to the primary English unit
--   ENGLISH, conversion of every MKS unit to corresponding primary English unit
--   CONVERT, conversion from primary English unit to other English units
--   CONVERT, conversion from MKS unit to other metric units
--
-- The overloaded function MKS returns the MKS value.
--
-- The overloaded function ENGLISH returns the primary English value.
--
-- The overloaded function CONVERT returns all units
--                                         other than MKS or primary English
-- 
-- Note: Any of these functions may need to be called with qualification.
--       Usually, MKS and ENGLISH will not require qualification.
--       Almost always, CONVERT will require qualification.
--
--         PUT ( LENGTH_CENTIMETER' ( CONVERT ( SOME_LENGTH ) ) ) ;
--
--      DO NOT get the concept of units conversion confused with the
--      Ada concept of type conversion. An Ada type conversion CAN NOT
--      change meters to feet.
--
with PHYSICAL_UNITS_BASIC ; use PHYSICAL_UNITS_BASIC ;
with PHYSICAL_UNITS_MECHANICAL ; use PHYSICAL_UNITS_MECHANICAL ;
with PHYSICAL_UNITS_ELECTRICAL ; use PHYSICAL_UNITS_ELECTRICAL ;

package PHYSICAL_UNITS_CONVERSION_ENGLISH is

  function ENGLISH ( ITEM : LENGTH_MKS ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_MIL ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_INCH ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_YARD ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_FATHOM ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_ROD ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_CHAIN_SURVEYOR ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_CHAIN_ENGINEER ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_FURLONG ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_MILE ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_MILE_NAUTICAL ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_LEAGUE_LAND ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_LEAGUE_MARINE ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : LENGTH_LIGHT_YEAR ) return LENGTH_ENGLISH ;

  function ENGLISH ( ITEM : MASS_MKS ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_POUND_TROY ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_GRAIN ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_PENNYWEIGHT_TROY ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_CARAT_TROY ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_SCRUPLE ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_DRAM_AVDP ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_OUNCE_AVDP ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_OUNCE_TROY ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_TON_SHORT ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : MASS_TON_LONG ) return MASS_ENGLISH ;

  function ENGLISH ( ITEM : TEMPERATURE_KELVIN ) return TEMPERATURE_FARENHEIT
      ;

  function ENGLISH ( ITEM : TEMPERATURE_CENTIGRADE ) return
      TEMPERATURE_FARENHEIT ;

  function ENGLISH ( ITEM : AREA_MKS ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : AREA_SQUARE_INCH ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : AREA_SQUARE_YARD ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : AREA_SQUARE_MILE ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : AREA_ACRE ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : AREA_CIRCULAR_MIL ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : AREA_HECTARE ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : AREA_TOWNSHIP ) return AREA_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_MKS ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_CUBIC_INCH ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_CUBIC_YARD ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_CUBIC_MILE ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_TEASPOON ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_TABLESPOON ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_OUNCE_FLUID ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_JIGGER ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_CUP ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_PINT_LIQUID ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_QUART_LIQUID ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_GALLON ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_KEG ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_BARREL ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_PINT_DRY ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_QUART_DRY ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_PECK ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_BUSHEL ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VOLUME_CORD ) return VOLUME_ENGLISH ;

  function ENGLISH ( ITEM : VELOCITY_MKS ) return VELOCITY_ENGLISH ;

  function ENGLISH ( ITEM : VELOCITY_INCHES_PER_SECOND ) return
      VELOCITY_ENGLISH ;

  function ENGLISH ( ITEM : VELOCITY_MILE_PER_HOUR ) return VELOCITY_ENGLISH ;

  function ENGLISH ( ITEM : VELOCITY_MILES_PER_SECOND ) return
      VELOCITY_ENGLISH ;

  function ENGLISH ( ITEM : VELOCITY_INCHES_PER_MINUTE ) return
      VELOCITY_ENGLISH ;

  function ENGLISH ( ITEM : VELOCITY_FEET_PER_MINUTE ) return VELOCITY_ENGLISH
      ;

  function ENGLISH ( ITEM : VELOCITY_MILES_PER_HOUR ) return VELOCITY_ENGLISH
      ;

  function ENGLISH ( ITEM : VELOCITY_KNOTS ) return VELOCITY_ENGLISH ;

  function ENGLISH ( ITEM : VELOCITY_FURLONG_PER_FORTNIGHT ) return
      VELOCITY_ENGLISH ;

  function ENGLISH ( ITEM : ACCELERATION_MKS ) return ACCELERATION_ENGLISH ;

  function ENGLISH ( ITEM : FORCE_MKS ) return FORCE_ENGLISH ;

  function ENGLISH ( ITEM : ENERGY_MKS ) return ENERGY_ENGLISH ;

  function ENGLISH ( ITEM : ENERGY_FOOT_POUND ) return ENERGY_ENGLISH ;

  function ENGLISH ( ITEM : ENERGY_KILOWATT_HOUR ) return ENERGY_ENGLISH ;

  function ENGLISH ( ITEM : ENERGY_HORSEPOWER_HOUR ) return ENERGY_ENGLISH ;

  function ENGLISH ( ITEM : POWER_HORSEPOWER_MECHANICAL ) return POWER_WATT ;

  function ENGLISH ( ITEM : POWER_HORSEPOWER_ELECTRICAL ) return POWER_WATT ;

  function ENGLISH ( ITEM : POWER_HORSEPOWER_BOILER ) return POWER_WATT ;

  function ENGLISH ( ITEM : POWER_B_T_U_PER_MINUTE ) return POWER_WATT ;

  function ENGLISH ( ITEM : POWER_B_T_U_PER_HOUR ) return POWER_WATT ;

  function ENGLISH ( ITEM : POWER_FOOT_POUND_PER_MINUTE ) return POWER_WATT ;

  function ENGLISH ( ITEM : POWER_FOOT_POUND_PER_SECOND ) return POWER_WATT ;

  function ENGLISH ( ITEM : DENSITY_MKS ) return DENSITY_ENGLISH ;

  function ENGLISH ( ITEM : FLOW_RATE_MKS ) return FLOW_RATE_ENGLISH ;

  function ENGLISH ( ITEM : FLOW_RATE_GALLON_PER_MINUTE ) return
      FLOW_RATE_ENGLISH ;

  function ENGLISH ( ITEM : FLOW_RATE_CUBIC_FEET_PER_MINUTE ) return
      FLOW_RATE_ENGLISH ;

  function ENGLISH ( ITEM : PRESSURE_MKS ) return PRESSURE_ENGLISH ;

  function ENGLISH ( ITEM : PRESSURE_TON_PER_SQUARE_FOOT ) return
      PRESSURE_ENGLISH ;

  function ENGLISH ( ITEM : PRESSURE_FEET_OF_WATER ) return PRESSURE_ENGLISH ;

  function ENGLISH ( ITEM : PRESSURE_INCHES_OF_MERCURY ) return
      PRESSURE_ENGLISH ;

end PHYSICAL_UNITS_CONVERSION_ENGLISH ;

